import numpy as np
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap


D = np.loadtxt( "datos.txt" )
print( D.shape )

X = D[:,0:2]
y = D[:,2]

# Separamos los datos en datos de entrenamiento y datos de prueba
X_entrena, X_prueba, y_entrena, y_prueba = train_test_split( X, y, random_state=4 )

Yj = np.array( [0.32142857, 0.35714286, 0.32142857] )
Vars = np.array( [[0.75664756, 0.98500829],
                  [0.01878853, 0.56731161],
                  [0.31819514, 0.86022389]] )

Medias = np.array( [[-2.90297501, -2.95837915],
                    [ 2.98083096, -2.906936  ],
                    [-3.06978751,  2.96582161]] )

def clasifica( vx ) :
	# vx es de tamaño 1X2
	# Tenemos que calcular las tres probabilidades
	# porque son tres clases y seleccionar la mayor
	vlnp = np.zeros( 3 )
	i = 0
	while i < 3 :
		# - ln sum pi*s^2 
		v = np.log( np.pi*np.sum( Vars[i] ) )
		# - ln (xi-mu)/(2*s^2)

		#   3x2      1x2
		w = Medias[i] - vx 
		v2 = w*w/(2.0*Vars[i])
		vlnp[i] = np.log( Yj[i] ) - v - np.sum(v2) 	
		i+=1

	return np.argmax( vlnp )

# Hacemos la gráfica de la superficie de decisión
# En x,y \in [-2,2]
n = 100
vx = np.linspace( -5, 4, n )
vy = np.linspace( -6, 6, n )

x, y = np.meshgrid( vx, vy )
Z = np.zeros( (n,n) )

va = np.zeros( (1,2) )

for i in range(n):
	for j in range(n):
		va[0,0]  = x[j,i]
		va[0,1]  = y[j,i]
		Z[j,i] = clasifica( va )


fig, ax = plt.subplots()

cm = plt.cm.RdBu
cm_bright = ListedColormap(['#FF0000', '#0000FF'])

ax.contourf( x, y, Z, cmap=cm, alpha=.8 )
ax.set_aspect(1)
# ax.scatter( decision.Vs[:,0], decision.Vs[:,1] )
ax.set_xticks(())
ax.set_yticks(())
ax.scatter( X_entrena[:,0], X_entrena[:,1], c=y_entrena, cmap=cm_bright, edgecolors='k' )
ax.scatter( X_prueba[:,0], X_prueba[:,1], marker='x', c=y_prueba, cmap=cm_bright, edgecolors='k' )

plt.show( )

